import sys
import tornado
import zmq
from tornado.web import Application
from tornado.ioloop import IOLoop

from tfw.config import WEB_PORT
from tfw.config.logs import logging
log = logging.getLogger(__name__)
from tfw.networking.zmq_websocket_handler import FSMManagingSocketHandler
from sql_injection_fsm import SQLInjectionFSM


class SQLInjectionFSMManagingSocketHandler(FSMManagingSocketHandler):
    def __init__(self, application, request, **kwargs):
        super().__init__(SQLInjectionFSM, application, request, **kwargs)


if __name__ == '__main__':
    routes = [
        (r'/ws', SQLInjectionFSMManagingSocketHandler),
    ]
    application = Application(
        routes,
        autoreload=True
    )

    application.listen(WEB_PORT)
    log.debug('Python version: {}'.format(sys.version[:5]))
    log.debug('Tornado version: {}'.format(tornado.version))
    log.debug('ZeroMQ version: {}'.format(zmq.zmq_version()))
    log.debug('PyZMQ version: {}'.format(zmq.pyzmq_version()))
    log.info('Tornado application listening on port {}'.format(WEB_PORT))
    IOLoop.instance().start()
